@@ -3,11 +3,12 @@ BEGIN {
   $App::Pocoirc::AUTHORITY = 'cpan:HINRIK';
 }
 BEGIN {
-  $App::Pocoirc::VERSION = '0.10';
+  $App::Pocoirc::VERSION = '0.11';
 }
 
 use strict;
 use warnings;
+use IO::Handle;
 use POE;
 use POSIX 'strftime';
 
@@ -19,10 +20,10 @@ sub new {
 sub run {
     my ($self) = @_;
 
-    if (defined $self->{cfg}{log_file}) {
-        open my $fh, '>>', $self->{cfg}{log_file}
-            or die "Can't open $self->{cfg}{log_file}: $!\n";
+    if (my $log = delete $self->{cfg}{log_file}) {
+        open my $fh, '>>', $log or die "Can't open $log: $!\n";
         close $fh;
+        $self->{log_file} = $log;
     }
 
     if (!$self->{no_color}) {
@@ -30,7 +31,7 @@ sub run {
         Term::ANSIColor->import();
     }
 
-    if ($self->{daemonize}) {
+    if ($self->{daemonize} && !$self->{check_cfg}) {
         require Proc::Daemon;
         eval { Proc::Daemon::Init->() };
         chomp $@;
@@ -73,14 +74,47 @@ sub run {
 sub _start {
     my ($kernel, $session, $self) = @_[KERNEL, SESSION, OBJECT];
 
-    $self->_status("Started");
-
     $kernel->sig(DIE => '_exception');
-    
+
     if (defined $self->{cfg}{lib} && @{ $self->{cfg}{lib} }) {
-        unshift @INC, @{ $self->{cfg}{lib} };
+        my $lib = delete $self->{cfg}{lib};
+        unshift @INC, @$lib;
     }
 
+    $self->_require_plugin($_) for @{ $self->{cfg}{global_plugins} || [] };
+
+    for my $opts (@{ $self->{cfg}{networks} }) {
+        $self->_require_plugin($_) for @{ $opts->{local_plugins} || [] };
+
+        die "Network name missing\n" if !defined $opts->{name};
+
+        if (!defined $opts->{server}) {
+            die "Server for network '$opts->{name}' not specified\n";
+        }
+
+        while (my ($opt, $value) = each %{ $self->{cfg} }) {
+            next if $opt =~ /^(?:networks|global_plugins|local_plugins)$/;
+            $opts->{$opt} = $value if !defined $opts->{$opt};
+        }
+
+        $opts->{class} = 'POE::Component::IRC::State' if !defined $opts->{class};
+        eval "require $opts->{class}";
+        chomp $@;
+        die "Can't load class $opts->{class}: $@\n" if $@;
+    }
+
+    # this can not be done earlier due to a bug in Perl 5.12 which causes
+    # the compilation of Net::DNS (used by POE::Component::IRC) to clear
+    # the signal handler
+    $kernel->sig(INT => '_exit');
+
+    if ($self->{check_cfg}) {
+        print "Config file is valid all modules could be compiled.\n";
+        return;
+    }
+
+    $self->_status("Started");
+
     # construct global plugins
     $self->_status("Constructing global plugins");
     $self->{global_plugs} = $self->_create_plugins(delete $self->{cfg}{global_plugins});
@@ -89,31 +123,17 @@ sub _start {
     for my $opts (@{ $self->{cfg}{networks} }) {
         my $network = delete $opts->{name};
         my $class = delete $opts->{class};
-        die "Network name missing\n" if !defined $network;
-        
-        while (my ($opt, $value) = each %{ $self->{cfg} }) {
-            $opts->{$opt} = $value if !defined $opts->{$opt};
-        }
-
-        if (!defined $opts->{server}) {
-            die "Server for network '$network' not specified\n";
-        }
         
         # construct network-specific plugins
         $self->_status('Constructing local plugins', $network);
         $self->{local_plugs}{$network} = $self->_create_plugins(delete $opts->{local_plugins});
 
-        $class = 'POE::Component::IRC::State' if !defined $class;
-        eval "require $class";
-        chomp $@;
-        die "Can't load class $class: $@" if $@;
-
         $self->_status('Spawning IRC component', $network);
         my $irc = $class->spawn(%$opts);
-
         push @{ $self->{ircs} }, [$network, $irc];
     }
 
+
     for my $entry (@{ $self->{ircs} }) {
         my ($network, $irc) = @$entry;
         
@@ -140,8 +160,6 @@ sub _start {
     delete $self->{global_plugs};
     delete $self->{local_plugs};
 
-    $kernel->sig(INT => '_exit');
-
     return;
 }
 
@@ -323,10 +341,10 @@ sub _status {
         }
     }
 
-    if (defined $self->{cfg}{log_file}) {
+    if (defined $self->{log_file}) {
         my $fh;
-        if (!open($fh, '>>:encoding(utf8)', $self->{cfg}{log_file}) && !$self->{daemonize}) {
-            warn "Can't open $self->{cfg}{log_file}: $!\n";
+        if (!open($fh, '>>:encoding(utf8)', $self->{log_file}) && !$self->{daemonize}) {
+            warn "Can't open $self->{log_file}: $!\n";
         }
 
         $fh->autoflush(1);
@@ -348,29 +366,39 @@ sub _irc_to_network {
     return;
 }
 
+# find out the canonical class name for the plugin and require() it
+sub _require_plugin {
+    my ($self, $plug_spec) = @_;
+
+    my ($class, $args) = @$plug_spec;
+    $args = {} if !defined $args;
+
+    my $fullclass = "POE::Component::IRC::Plugin::$class";
+    my $canonclass = $fullclass;
+    my $error;
+    eval "require $fullclass";
+    if ($@) {
+        $error .= $@;
+        eval "require $class";
+        if ($@) {
+            chomp $@;
+            $error .= $@;
+            die "Failed to load plugin $class or $fullclass: $error\n";
+        }
+        $canonclass = $class;
+    }
+
+    $plug_spec->[1] = $args;
+    $plug_spec->[2] = $canonclass;
+    return;
+}
+
 sub _create_plugins {
     my ($self, $plugins) = @_;
 
     my @return;
-    for my $plugin (@$plugins) {
-        my ($class, $args) = @$plugin;
-        $args = {} if !defined $args;
-
-        my $fullclass = "POE::Component::IRC::Plugin::$class";
-        my $canonclass = $fullclass;
-        my $error;
-        eval "require $fullclass";
-        if ($@) {
-            $error .= $@;
-            eval "require $class";
-            if ($@) {
-                chomp $@;
-                $error .= $@;
-                die "Failed to load plugin $class or $fullclass: $error\n";
-            }
-            $canonclass = $class;
-        }
-
+    for my $plug_spec (@$plugins) {
+        my ($class, $args, $canonclass) = @$plug_spec;
         my $obj = $canonclass->new(%$args);
         push @return, [$class, $obj];
     }
@@ -410,7 +438,7 @@ sub _shutdown {
             my ($network, $obj) = @$irc;
             $obj->connected
                 ? $obj->yield(quit => $reason)
-                : $obj->shutdown();
+                : $obj->yield('shutdown');
         }
         $self->{shutdown} = 1;
     }
@@ -469,8 +497,9 @@ to load locally (one object per component) or globally (single object)
 
 The configuration file is in L<YAML|YAML> or L<JSON|JSON> format. It consists
 of a hash containing C<global_plugins>, C<local_plugins>, C<networks>, C<lib>,
-C<log_file>, and default parameters to POE::Component::IRC. Only C<networks>
-is required.
+C<log_file>, and default parameters to
+L<POE::Component::IRC|POE::Component::IRC/spawn>. Only C<networks> is
+required.
 
 C<lib> is an array of directories containing Perl modules (e.g. plugins).
 Just like Perl's I<-I>.
